# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/pheno-utils',
                'doc_host': 'https://pheno-ai.github.io',
                'git_url': 'https://github.com/pheno-ai/pheno-utils',
                'lib_path': 'pheno_utils'},
  'syms': { 'pheno_utils.age_reference_plots': { 'pheno_utils.age_reference_plots.AgeRefPlot': ( 'age_reference_plots.html#agerefplot',
                                                                                                 'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.__init__': ( 'age_reference_plots.html#agerefplot.__init__',
                                                                                                          'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.calc_linear_fit': ( 'age_reference_plots.html#agerefplot.calc_linear_fit',
                                                                                                                 'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot': ( 'age_reference_plots.html#agerefplot.plot',
                                                                                                      'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot_agehist': ( 'age_reference_plots.html#agerefplot.plot_agehist',
                                                                                                              'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot_ornaments': ( 'age_reference_plots.html#agerefplot.plot_ornaments',
                                                                                                                'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot_percentiles': ( 'age_reference_plots.html#agerefplot.plot_percentiles',
                                                                                                                  'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot_scatter': ( 'age_reference_plots.html#agerefplot.plot_scatter',
                                                                                                              'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.AgeRefPlot.plot_valhist': ( 'age_reference_plots.html#agerefplot.plot_valhist',
                                                                                                              'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.GenderAgeRefPlot': ( 'age_reference_plots.html#genderagerefplot',
                                                                                                       'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.GenderAgeRefPlot.__init__': ( 'age_reference_plots.html#genderagerefplot.__init__',
                                                                                                                'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.GenderAgeRefPlot.plot': ( 'age_reference_plots.html#genderagerefplot.plot',
                                                                                                            'pheno_utils/age_reference_plots.py'),
                                                 'pheno_utils.age_reference_plots.get_gam_expectiles': ( 'age_reference_plots.html#get_gam_expectiles',
                                                                                                         'pheno_utils/age_reference_plots.py')},
            'pheno_utils.basic_analysis': { 'pheno_utils.basic_analysis.assign_nearest_research_stage': ( 'basic_analysis.html#assign_nearest_research_stage',
                                                                                                          'pheno_utils/basic_analysis.py'),
                                            'pheno_utils.basic_analysis.custom_describe': ( 'basic_analysis.html#custom_describe',
                                                                                            'pheno_utils/basic_analysis.py')},
            'pheno_utils.basic_plots': { 'pheno_utils.basic_plots.data_ecdfplot': ( 'basic_plots.html#data_ecdfplot',
                                                                                    'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.data_histplot': ( 'basic_plots.html#data_histplot',
                                                                                    'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.hist_ecdf_plots': ( 'basic_plots.html#hist_ecdf_plots',
                                                                                      'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.plot_data_collection': ( 'basic_plots.html#plot_data_collection',
                                                                                           'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.plot_hist_stats': ( 'basic_plots.html#plot_hist_stats',
                                                                                      'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.plot_stats': ( 'basic_plots.html#plot_stats',
                                                                                 'pheno_utils/basic_plots.py'),
                                         'pheno_utils.basic_plots.show_fundus': ( 'basic_plots.html#show_fundus',
                                                                                  'pheno_utils/basic_plots.py')},
            'pheno_utils.blandaltman_plots': { 'pheno_utils.blandaltman_plots.bland_altman_triple_plot': ( 'blandaltman_plots.html#bland_altman_triple_plot',
                                                                                                           'pheno_utils/blandaltman_plots.py')},
            'pheno_utils.cgm_plots': { 'pheno_utils.cgm_plots.AGP': ('cgm_plots.html#agp', 'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.AGP.__init__': ('cgm_plots.html#agp.__init__', 'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.AGP.plot': ('cgm_plots.html#agp.plot', 'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot': ('cgm_plots.html#cgmplot', 'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.__init__': ( 'cgm_plots.html#cgmplot.__init__',
                                                                                   'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.plot': ('cgm_plots.html#cgmplot.plot', 'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.plot_diet': ( 'cgm_plots.html#cgmplot.plot_diet',
                                                                                    'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.plot_gluc': ( 'cgm_plots.html#cgmplot.plot_gluc',
                                                                                    'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.plot_sleep': ( 'cgm_plots.html#cgmplot.plot_sleep',
                                                                                     'pheno_utils/cgm_plots.py'),
                                       'pheno_utils.cgm_plots.CGMPlot.prep_plot': ( 'cgm_plots.html#cgmplot.prep_plot',
                                                                                    'pheno_utils/cgm_plots.py')},
            'pheno_utils.cohort_selector': { 'pheno_utils.cohort_selector.CohortSelector': ( 'cohort_selector.html#cohortselector',
                                                                                             'pheno_utils/cohort_selector.py'),
                                             'pheno_utils.cohort_selector.CohortSelector.__init__': ( 'cohort_selector.html#cohortselector.__init__',
                                                                                                      'pheno_utils/cohort_selector.py'),
                                             'pheno_utils.cohort_selector.CohortSelector.select': ( 'cohort_selector.html#cohortselector.select',
                                                                                                    'pheno_utils/cohort_selector.py')},
            'pheno_utils.config': { 'pheno_utils.config.copy_tre_config': ('config.html#copy_tre_config', 'pheno_utils/config.py'),
                                    'pheno_utils.config.generate_synthetic_data': ( 'config.html#generate_synthetic_data',
                                                                                    'pheno_utils/config.py'),
                                    'pheno_utils.config.generate_synthetic_data_like': ( 'config.html#generate_synthetic_data_like',
                                                                                         'pheno_utils/config.py')},
            'pheno_utils.config_setup.create_default_config': {},
            'pheno_utils.dates_plots': { 'pheno_utils.dates_plots.dates_dist_plot': ( 'date_plots.html#dates_dist_plot',
                                                                                      'pheno_utils/dates_plots.py')},
            'pheno_utils.ecg_analysis': { 'pheno_utils.ecg_analysis.get_hrv_df': ( 'ecg_analysis.html#get_hrv_df',
                                                                                   'pheno_utils/ecg_analysis.py'),
                                          'pheno_utils.ecg_analysis.vis_ecg': ('ecg_analysis.html#vis_ecg', 'pheno_utils/ecg_analysis.py')},
            'pheno_utils.meta_loader': { 'pheno_utils.meta_loader.MetaLoader': ( 'meta_loader.html#metaloader',
                                                                                 'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__concat__': ( 'meta_loader.html#metaloader.__concat__',
                                                                                            'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__get_dataset_path__': ( 'meta_loader.html#metaloader.__get_dataset_path__',
                                                                                                      'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__getitem__': ( 'meta_loader.html#metaloader.__getitem__',
                                                                                             'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__init__': ( 'meta_loader.html#metaloader.__init__',
                                                                                          'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__load_dictionaries__': ( 'meta_loader.html#metaloader.__load_dictionaries__',
                                                                                                       'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__repr__': ( 'meta_loader.html#metaloader.__repr__',
                                                                                          'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.__str__': ( 'meta_loader.html#metaloader.__str__',
                                                                                         'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.get': ( 'meta_loader.html#metaloader.get',
                                                                                     'pheno_utils/meta_loader.py'),
                                         'pheno_utils.meta_loader.MetaLoader.load': ( 'meta_loader.html#metaloader.load',
                                                                                      'pheno_utils/meta_loader.py')},
            'pheno_utils.pheno_loader': { 'pheno_utils.pheno_loader.PhenoLoader': ( 'pheno_loader.html#phenoloader',
                                                                                    'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__concat__': ( 'pheno_loader.html#phenoloader.__concat__',
                                                                                               'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__get_dataset_path__': ( 'pheno_loader.html#phenoloader.__get_dataset_path__',
                                                                                                         'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__get_dictionary_file_path__': ( 'pheno_loader.html#phenoloader.__get_dictionary_file_path__',
                                                                                                                 'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__get_file_path__': ( 'pheno_loader.html#phenoloader.__get_file_path__',
                                                                                                      'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__getitem__': ( 'pheno_loader.html#phenoloader.__getitem__',
                                                                                                'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__init__': ( 'pheno_loader.html#phenoloader.__init__',
                                                                                             'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__load_age_sex__': ( 'pheno_loader.html#phenoloader.__load_age_sex__',
                                                                                                     'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__load_dataframes__': ( 'pheno_loader.html#phenoloader.__load_dataframes__',
                                                                                                        'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__load_dictionary__': ( 'pheno_loader.html#phenoloader.__load_dictionary__',
                                                                                                        'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__load_one_dataframe__': ( 'pheno_loader.html#phenoloader.__load_one_dataframe__',
                                                                                                           'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__repr__': ( 'pheno_loader.html#phenoloader.__repr__',
                                                                                             'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.__str__': ( 'pheno_loader.html#phenoloader.__str__',
                                                                                            'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.describe_field': ( 'pheno_loader.html#phenoloader.describe_field',
                                                                                                   'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.get': ( 'pheno_loader.html#phenoloader.get',
                                                                                        'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.load_sample_data': ( 'pheno_loader.html#phenoloader.load_sample_data',
                                                                                                     'pheno_utils/pheno_loader.py'),
                                          'pheno_utils.pheno_loader.PhenoLoader.replace_bulk_data_path': ( 'pheno_loader.html#phenoloader.replace_bulk_data_path',
                                                                                                           'pheno_utils/pheno_loader.py')},
            'pheno_utils.questionnaires_handler': { 'pheno_utils.questionnaires_handler.convert_to_string': ( 'questionnaire_handler.html#convert_to_string',
                                                                                                              'pheno_utils/questionnaires_handler.py'),
                                                    'pheno_utils.questionnaires_handler.tranform_answers': ( 'questionnaire_handler.html#tranform_answers',
                                                                                                             'pheno_utils/questionnaires_handler.py')},
            'pheno_utils.sleep_plots': { 'pheno_utils.sleep_plots.format_xticks': ( 'sleep_plots.html#format_xticks',
                                                                                    'pheno_utils/sleep_plots.py'),
                                         'pheno_utils.sleep_plots.get_legend_colors': ( 'sleep_plots.html#get_legend_colors',
                                                                                        'pheno_utils/sleep_plots.py'),
                                         'pheno_utils.sleep_plots.plot_channels': ( 'sleep_plots.html#plot_channels',
                                                                                    'pheno_utils/sleep_plots.py'),
                                         'pheno_utils.sleep_plots.plot_events': ( 'sleep_plots.html#plot_events',
                                                                                  'pheno_utils/sleep_plots.py'),
                                         'pheno_utils.sleep_plots.plot_sleep': ( 'sleep_plots.html#plot_sleep',
                                                                                 'pheno_utils/sleep_plots.py')}}}
